/***************************************************************************
 *                                                                         *
 *   bibliography.cpp								   *
 *									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Read the file COPYING for details.					   *
 *									   *
 *   copyright: (C) 2003 by Jan Mueller					   *
 *   email: janmueller7@hotmail.com					   *
 *									   *
 ***************************************************************************/

#include "bibliography.h"
#include "liturl.h"
#include <qlayout.h>
#include <qtoolbar.h>
#include <qaction.h>
#include <qpixmap.h>
#include <qmessagebox.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qapplication.h>
#include <qsqldatabase.h>
#include <qsqlcursor.h>
#include <qevent.h>
#include <qlayout.h>
#include <qsettings.h>
#include <qtextedit.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include "error.h"
#include "global.h"
#include "mainwindowimpl.h"
#include "bibliographyform.h"
#include "urlevent.h"
#include "selectitem.h"
#include "bibtextypes.h"
#include "bibtexfactory.h"

Bibliography::Bibliography(LitUrl url, QObject *receiver, QString slot, MainWindow *parent, const char *name): ReadWriteFormBase(url, receiver, slot, parent, name), toolbar(0L) {
  biblioform=new BibliographyForm(this, "biblioform");
  QHBoxLayout *l=new QHBoxLayout(this);
  l->addWidget(biblioform);
  initToolBar();
  connect(biblioform->choose_org_publisher, SIGNAL(clicked()), this, SLOT(chooseOrgPublisher()));
  connect(biblioform->genbibtex, SIGNAL(clicked()), this, SLOT(generateBibTex()));
  QSettings settings;
  QString backgr = settings.readEntry(QString("/literature/backgr_")+name, "");    if(!backgr.isEmpty()) biblioform->setErasePixmap(QPixmap(backgr));
  setData();
}

void Bibliography::initToolBar() {
  toolbar=new QToolBar(mainWindow());
  
  showPublAction=new QAction("Show Publication", QIconSet(loadPixmap("showpubl.png")), "Show Publication", 0, this, "showpublication");
  connect(showPublAction, SIGNAL(activated()), this, SLOT(showPubl()));
  showPublAction->addTo(toolbar);
  toolbar->show();
}

Bibliography::~Bibliography() {
  write();
  delete toolbar;
  
}

void Bibliography::insertBibtexField(QString label, QString field, QFrame *frame, QSqlForm *form) {
  QLabel *l=new QLabel(frame);
  l->setText(label);
  QLineEdit *e = new QLineEdit(frame);
  form->insert(e, field);
}
 
bool Bibliography::setData()
{
  try {
    ReadWriteFormBase::setData();
    LitUrl u=url();
    setWorkingGui(true);
    form.clear();
    formcursor.clear();
    formcursor.setMode(QSqlCursor::Writable);
    formcursor.setName("publication");
    if(!formcursor.select(u.query()))
      throw(Error(formcursor.lastError()));
    if(formcursor.size()!=1) throw Error("Invalid Record!");
    formcursor.first();
    QString no=formcursor.value("no").toString();
    form.setRecord(formcursor.primeUpdate());
    form.insert(biblioform->title, "title");
    form.insert(biblioform->subtitle, "subtitle");
    form.insert(biblioform->translator, "translator");
    form.insert(biblioform->orgyear, "orgyear");
    form.insert(biblioform->orgtitle, "orgtitle");
    form.insert(biblioform->bibtex, "bibtex");
    form.insert(biblioform->isbn, "isbn_issn");
    form.insert(biblioform->hidingplace, "hidingplace");
    form.insert(biblioform->pages, "pages");
    form.insert(biblioform->modified, "modified");
    form.insert(biblioform->created, "created");
    int bibtextype=formcursor.value("entrytype").toInt();

    QGridLayout *layout=new QGridLayout(biblioform->frame, 4, 6);
    layout->setAutoAdd(true);
    layout->setColSpacing(0, 60);
    layout->setColSpacing(1, 80);
    layout->setColSpacing(2, 60);
    layout->setColSpacing(3, 80);
    layout->setColSpacing(4, 60);
    layout->setColSpacing(5, 80);

    layout->setColStretch(0, 0);
    layout->setColStretch(1, 1);
    layout->setColStretch(2, 0);
    layout->setColStretch(3, 1);
    layout->setColStretch(4, 0);
    layout->setColStretch(5, 1);
    layout->setSpacing(5);


    switch(bibtextype) {
    case BT_BOOK:
    case BT_INBOOK:
    case BT_INCOLLECTION:
      insertBibtexField("Editor ", "editor", biblioform->frame, &form);
      insertBibtexField("Type ", "type", biblioform->frame, &form);
      insertBibtexField("Series ", "series", biblioform->frame, &form);
      insertBibtexField("Volume/Number ", "volume", biblioform->frame, &form);
      insertBibtexField("Month ", "month", biblioform->frame, &form);
      insertBibtexField("Edition ", "edition", biblioform->frame, &form);
      break;
    case BT_BOOKLET:
    case BT_MISC:
      insertBibtexField("How Published ", "howpublished", biblioform->frame, &form);
      insertBibtexField("Month ", "month", biblioform->frame, &form);
      break;
    case BT_JOURNAL:
    case BT_ARTICLE:
      insertBibtexField("Number ", "number", biblioform->frame, &form);
      insertBibtexField("Volume ", "volume", biblioform->frame, &form);
      insertBibtexField("Month ", "month", biblioform->frame, &form);
      break;
    case BT_MANUAL:
      insertBibtexField("Organisation ", "organization", biblioform->frame, &form);
      insertBibtexField("Edition ", "edition", biblioform->frame, &form);
      insertBibtexField("Month ", "month", biblioform->frame, &form);
      break;
    case BT_PROCEEDING:
    case BT_INPROCEEDING:
      insertBibtexField("Editor ", "editor", biblioform->frame, &form);
      insertBibtexField("Organisation ", "organization", biblioform->frame, &form);
      insertBibtexField("Series ", "series", biblioform->frame, &form);
      insertBibtexField("Volume/Number ", "volume", biblioform->frame, &form);
      insertBibtexField("Month ", "month", biblioform->frame, &form);
      break;
    case BT_TECHREPORT:
      insertBibtexField("Type ", "type", biblioform->frame, &form);
      insertBibtexField("Month ", "month", biblioform->frame, &form);
      insertBibtexField("Number ", "number", biblioform->frame, &form);
      break;
    case BT_UNPUBLISHED:
      insertBibtexField("Month ", "month", biblioform->frame, &form);
      break;
    case BT_PHDTHESIS:
    case BT_MASTERTHESIS:
      insertBibtexField("Type ", "type", biblioform->frame, &form);
      insertBibtexField("Month ", "month", biblioform->frame, &form);
      insertBibtexField("School ", "school", biblioform->frame, &form);
      break;
    }

    form.readFields();

    QString orgpublisher=formcursor.value("orgpublisher_no").toString();
    QSqlQuery query("select publisher.name from publisher where no="+orgpublisher);
    if(query.first())
      biblioform->orgpublisher->setText(query.value(0).toString());

    
    setValid(true);
  } catch(Error &e) {
    setValid(false);
    setReadonlyGui(true);
    QMessageBox::warning(this, "Error", e.message());
    return false;
  }
  setValid(true);
  return true;
}

void Bibliography::chooseOrgPublisher() {
  SelectItem *s=new SelectItem(this, "selorgpublisher");
  Q_ASSERT(s);
  s->setData("publisher/publisher", "select no, name from publisher order by name");
  if(s->exec()==QDialog::Accepted) {
    QSqlRecord *rec=formcursor.primeUpdate();
    rec->setValue("orgpublisher_no", no(s->url()));
    biblioform->orgpublisher->setText(s->text());
  }
}

void Bibliography::generateBibTex() {
  // FixMe
  //  write();
  //  setData();
  PublBibTexFactory f(formcursor);
  biblioform->bibtex->setText(f.getEntry());
}

void Bibliography::setWorkingGui(bool work) {
  ReadWriteFormBase::setWorkingGui(work);
}

void Bibliography::setReadonlyGui(bool ro) {
  debug("Bibliography::setReadonlyGui");
  ReadWriteFormBase::setReadonlyGui(ro);    
  
  form.insert(biblioform->subtitle, "subtitle");
  form.insert(biblioform->translator, "translator");
  form.insert(biblioform->orgyear, "orgyear");
  form.insert(biblioform->orgtitle, "orgtitle");
  form.insert(biblioform->bibtex, "bibtex");
  form.insert(biblioform->isbn, "isbn");
  form.insert(biblioform->hidingplace, "hidingplace");
  form.insert(biblioform->pages, "pages");	    
  
  
  biblioform->subtitle->setEnabled(!ro);
  biblioform->translator->setEnabled(!ro);
  biblioform->orgyear->setEnabled(!ro);
  biblioform->orgtitle->setEnabled(!ro);
  biblioform->bibtex->setEnabled(!ro);
  biblioform->isbn->setEnabled(!ro);
  biblioform->hidingplace->setEnabled(!ro);
  biblioform->pages->setEnabled(!ro);    
}

void Bibliography::setInvalidGui(bool inv) {
  ReadWriteFormBase::setInvalidGui(inv);
  setWorkingGui(false);
  setReadonlyGui(inv);
}

void Bibliography::showPubl() {
  qApp->postEvent(topLevelWidget(), new UrlEvent("lit://publication/publication?no="+formcursor.value("no").toString(), false));
}
