/***************************************************************************
 *                                                                         *
 *   editorhelper.cpp								   *
 *									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Read the file COPYING for details.					   *
 *									   *
 *   copyright: (C) 2003 by Jan Mueller					   *
 *   email: janmueller7@hotmail.com					   *
 *									   *
 ***************************************************************************/

#include "editorhelper.h"
#include "global.h"
#include "readwriteformbase.h"
#include <qfontdatabase.h>
#include <qcolordialog.h>
#include <qaction.h>
#include <qcolor.h>
#include <qtoolbar.h>
#include <qcombobox.h>
#include <qapplication.h>
#include <qlineedit.h>
#include <qstylesheet.h>
#include <qregexp.h>
#include <qcolor.h>

EditorHelper::EditorHelper(ReadWriteFormBase *parent, QTextEdit *editor):
	QObject(parent), parent(parent), e(editor) {
    setupToolBar();
    setupEditorConnections();
 };

EditorHelper::~EditorHelper() {
    delete toolbar;
};



  /* The following code is copied from the qt-e example and slightly modified. */

void EditorHelper::setupEditorConnections() {
  connect( e, SIGNAL( currentFontChanged( const QFont & ) ),
           this, SLOT( fontChanged( const QFont & ) ) );
  connect( e, SIGNAL( currentColorChanged( const QColor & ) ),
           this, SLOT( colorChanged( const QColor & ) ) );
  connect( e, SIGNAL( currentAlignmentChanged( int ) ),
           this, SLOT( alignmentChanged( int ) ) );
  e->setTextFormat(Qt::RichText);
  e->styleSheet()->item("p")->setMargin(QStyleSheetItem::MarginVertical, 0);
//  e->styleSheet()->item("p")->setWhiteSpaceMode(QStyleSheetItem::WhiteSpacePre);
//  e->styleSheet()->item("qt")->setWhiteSpaceMode(QStyleSheetItem::WhiteSpacePre);
}

void EditorHelper::setupToolBar() {
  Q_ASSERT(parent);
  toolbar=new QToolBar(parent->mainWindow());
  toolbar->setLabel( "Edit Actions" );
  
  QAction *a = new QAction("Undo", QIconSet(loadPixmap("editundo.png" )), "&Undo", CTRL + Key_Z, this, "editUndo" );
  connect( a, SIGNAL( activated() ), this, SLOT( editUndo() ) );
  a->addTo(toolbar );
  
  a = new QAction("Redo", QIconSet( loadPixmap("editredo.png" )), "&Redo", CTRL + Key_Y, this, "editRedo" );
  connect( a, SIGNAL( activated() ), this, SLOT( editRedo() ) );
  a->addTo( toolbar );
  
  comboStyle = new QComboBox( FALSE, toolbar );
  comboStyle->insertItem( "Standard" );
  comboStyle->insertItem( "Bullet List (Disc)" );
  comboStyle->insertItem( "Bullet List (Circle)" );
  comboStyle->insertItem( "Bullet List (Square)" );
  comboStyle->insertItem( "Ordered List (Decimal)" );
  comboStyle->insertItem( "Ordered List (Alpha lower)" );
  comboStyle->insertItem( "Ordered List (Alpha upper)" );
  connect( comboStyle, SIGNAL( activated( int ) ),
    this, SLOT( textStyle( int ) ) );

  comboFont = new QComboBox( TRUE, toolbar );
  QFontDatabase db;
  comboFont->insertStringList( db.families() );
  connect( comboFont, SIGNAL( activated( const QString & ) ),
    this, SLOT( textFamily( const QString & ) ) );
  comboFont->lineEdit()->setText( QApplication::font().family() );

  comboSize = new QComboBox( TRUE, toolbar );
  QValueList<int> sizes = db.standardSizes();
  QValueList<int>::Iterator it = sizes.begin();
  for ( ; it != sizes.end(); ++it )
    comboSize->insertItem( QString::number( *it ) );

  connect( comboSize, SIGNAL( activated( const QString & ) ),
           this, SLOT( textSize( const QString & ) ) );
  comboSize->lineEdit()->setText( QString::number( QApplication::font().pointSize() ) );

  actionTextBold = new QAction( "Bold", QIconSet(loadPixmap("textbold.png" )), "&Bold", CTRL + Key_B, this, "textBold" );
  connect( actionTextBold, SIGNAL( activated() ), this, SLOT( textBold() ) );
  actionTextBold->addTo( toolbar );
  actionTextBold->setToggleAction( TRUE );
  
  actionTextItalic = new QAction( "Italic", QIconSet(loadPixmap( "textitalic.png" )), "&Italic", CTRL + Key_I, this, "textItalic" );
  connect( actionTextItalic, SIGNAL( activated() ), this, SLOT( textItalic() ) );
  actionTextItalic->addTo( toolbar );
  actionTextItalic->setToggleAction( TRUE );
  
  actionTextUnderline = new QAction( "Underline", QIconSet(loadPixmap("textunder.png" )), "&Underline", CTRL + Key_U, this, "textUnderline" );
  connect( actionTextUnderline, SIGNAL( activated() ), this, SLOT( textUnderline() ) );
  actionTextUnderline->addTo( toolbar );
  actionTextUnderline->setToggleAction( TRUE );

  QActionGroup *grp = new QActionGroup( this );
  grp->setExclusive( TRUE );
  connect( grp, SIGNAL( selected( QAction* ) ), this, SLOT( textAlign( QAction* ) ) );

  actionAlignLeft = new QAction( "Left", QIconSet(loadPixmap("textleft.png")), "&Left", CTRL + Key_L, grp, "textLeft" );
  actionAlignLeft->addTo( toolbar );
  actionAlignLeft->setToggleAction( TRUE );
  
  actionAlignCenter = new QAction("Center", QIconSet(loadPixmap("textcenter.png")), "C&enter", CTRL + Key_E, grp, "textCenter" );
  actionAlignCenter->addTo( toolbar );
  actionAlignCenter->setToggleAction( TRUE );
  
  actionAlignRight = new QAction( "Right", QIconSet(loadPixmap("textright.png")), "&Right", CTRL + Key_R, grp, "textRight" );
  actionAlignRight->addTo( toolbar );
  actionAlignRight->setToggleAction( TRUE );
  
  actionAlignJustify = new QAction( "Justify", QIconSet(loadPixmap("textjustify.png")), "&Justify", CTRL + Key_J, grp, "textjustify" );
  actionAlignJustify->addTo( toolbar );
  actionAlignJustify->setToggleAction( TRUE );

  QPixmap pix( 16, 16 );
  pix.fill( black );
  actionTextColor = new QAction( "Color", pix, "&Color...", 0, this, "textColor" );
  connect( actionTextColor, SIGNAL( activated() ), this, SLOT( textColor() ) );
  actionTextColor->addTo( toolbar );
}


void EditorHelper::editUndo()
{
    if ( !e )
	return;
    e->undo();
}

void EditorHelper::editRedo()
{
    if ( !e )
	return;
    e->redo();
}

void EditorHelper::editCut()
{
    if ( !e )
	return;
    e->cut();
}

void EditorHelper::editCopy()
{
    if ( !e )
	return;
    e->copy();
}

void EditorHelper::editPaste()
{
    if ( !e )
	return;
    e->paste();
}

void EditorHelper::textBold()
{
    if ( !e )
	return;
    e->setBold( actionTextBold->isOn() );
}

void EditorHelper::textUnderline()
{
    if ( !e )
	return;
    e->setUnderline( actionTextUnderline->isOn() );
}

void EditorHelper::textItalic()
{
    if ( !e )
	return;
    e->setItalic( actionTextItalic->isOn() );
}

void EditorHelper::textFamily( const QString &f )
{
    if ( !e )
	return;
    e->setFamily( f );
    e->viewport()->setFocus();
}

void EditorHelper::textSize( const QString &p )
{
    if ( !e )
	return;
    e->setPointSize( p.toInt() );
    e->viewport()->setFocus();
}

void EditorHelper::textStyle( int i )
{
    if ( !e )
	return;
    if ( i == 0 )
  e->setParagType( QStyleSheetItem::DisplayBlock, QStyleSheetItem::ListDisc );
    else if ( i == 1 )
	e->setParagType( QStyleSheetItem::DisplayListItem, QStyleSheetItem::ListDisc );
    else if ( i == 2 )
	e->setParagType( QStyleSheetItem::DisplayListItem, QStyleSheetItem::ListCircle );
    else if ( i == 3 )
	e->setParagType( QStyleSheetItem::DisplayListItem, QStyleSheetItem::ListSquare );
    else if ( i == 4 )
	e->setParagType( QStyleSheetItem::DisplayListItem, QStyleSheetItem::ListDecimal );
    else if ( i == 5 )
	e->setParagType( QStyleSheetItem::DisplayListItem, QStyleSheetItem::ListLowerAlpha );
    else if ( i == 6 )
	e->setParagType( QStyleSheetItem::DisplayListItem, QStyleSheetItem::ListUpperAlpha );
    e->viewport()->setFocus();
}

void EditorHelper::textColor()
{
    if ( !e )
	return;
    QColor col = QColorDialog::getColor( e->color(), parent );
    if ( !col.isValid() )
	return;
    e->setColor( col );
    QPixmap pix( 16, 16 );
    pix.fill( black );
    actionTextColor->setIconSet( pix );
}

void EditorHelper::textAlign( QAction *a )
{
    if ( !e )
	return;
    if ( a == actionAlignLeft )
	e->setAlignment( AlignLeft );
    else if ( a == actionAlignCenter )
	e->setAlignment( AlignHCenter );
    else if ( a == actionAlignRight )
	e->setAlignment( AlignRight );
    else if ( a == actionAlignJustify )
	e->setAlignment( AlignJustify );
}

void EditorHelper::fontChanged( const QFont &f )
{
    comboFont->lineEdit()->setText( f.family() );
    comboSize->lineEdit()->setText( QString::number( f.pointSize() ) );
    actionTextBold->setOn( f.bold() );
    actionTextItalic->setOn( f.italic() );
    actionTextUnderline->setOn( f.underline() );
}

void EditorHelper::colorChanged( const QColor &c )
{
    QPixmap pix( 16, 16 );
    pix.fill( c );
    actionTextColor->setIconSet( pix );
}

void EditorHelper::alignmentChanged( int a )
{
    if ( ( a == AlignAuto ) || ( a & AlignLeft ))
	actionAlignLeft->setOn( TRUE );
    else if ( ( a & AlignHCenter ) )
	actionAlignCenter->setOn( TRUE );
    else if ( ( a & AlignRight ) )
	actionAlignRight->setOn( TRUE );
    else if ( ( a & AlignJustify ) )
	actionAlignJustify->setOn( TRUE );
}
