/***************************************************************************
 *                                                                         *
 *   project.cpp								   *
 *									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Read the file COPYING for details.					   *
 *									   *
 *   copyright: (C) 2003 by Jan Mueller					   *
 *   email: janmueller7@hotmail.com					   *
 *									   *
 ***************************************************************************/

#include "project.h"
#include "liturl.h"
#include <qlayout.h>
#include <qtoolbar.h>
#include <qaction.h>
#include <qpixmap.h>
#include <qmessagebox.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qapplication.h>
#include <qsqldatabase.h>
#include <qsqlcursor.h>
#include <qevent.h>
#include <qsettings.h>
#include <qtextedit.h>
#include "error.h"
#include "global.h"
#include "mainwindowimpl.h"
#include "projectform.h"
#include "selectitem.h"
#include "generatebibtex.h"
#include "filllinklistbox.h"
#include "linklistbox.h"
#include "selectpart.h"

Project::Project(LitUrl url, QObject *receiver, QString slot, MainWindow *parent, const char *name): ReadWriteFormBase(url, receiver, slot, parent, name) {    
  projectform=new ProjectForm(this, "projectform");
  QHBoxLayout *l=new QHBoxLayout(this);
  l->addWidget(projectform);
  
  QSettings settings;
  QString backgr = settings.readEntry(QString("/literature/backgr_")+name, "");
  if(!backgr.isEmpty()) projectform->setErasePixmap(QPixmap(backgr));
  
  initToolBar();
  setData();
}


void Project::initToolBar() {
  toolbar=new QToolBar(mainWindow());
  
  insertPublAction=new QAction("Insert Publication", QIconSet(loadPixmap("inspubl.png")), "Insert Publication", 0, this, "insertpublication");
  connect(insertPublAction, SIGNAL(activated()), this, SLOT(insertPublication()));
  
  deletePublAction=new QAction("Delete Publication", QIconSet(loadPixmap("delpubl.png")), "Delete Publication", 0, this, "deletepublication");
  connect(deletePublAction, SIGNAL(activated()), this, SLOT(deletePublication()));
  
  insertPartAction=new QAction("Insert Part", QIconSet(loadPixmap("inspart.png")), "Insert Part", 0, this, "insertpart");
  connect(insertPartAction, SIGNAL(activated()), this, SLOT(insertPart()));
  
  generateBibTexAction=new QAction("Generate BibTex", QIconSet(loadPixmap("tex.png")), "Generate BibTex", 0, this, "generatebibtex");
  connect(generateBibTexAction, SIGNAL(activated()), this, SLOT(generateBibTex()));
  
  insertPublAction->addTo(toolbar);
  insertPartAction->addTo(toolbar);
  deletePublAction->addTo(toolbar);
  generateBibTexAction->addTo(toolbar);
  toolbar->show();

  insertPublAction->addTo(roMenu());
  insertPartAction->addTo(roMenu());
  deletePublAction->addTo(roMenu());
  generateBibTexAction->addTo(roMenu());

}


Project::~Project() {
    write();
    delete toolbar;
}

bool Project::setData()
{
    try {
	ReadWriteFormBase::setData();
	LitUrl u=url();
	setWorkingGui(true);
	
	form.clear();
	formcursor.clear();
	formcursor.setMode(QSqlCursor::Writable);
	formcursor.setName("project");
	if(!formcursor.select(u.query()))
	    throw(Error(formcursor.lastError()));
	if(formcursor.size()!=1) throw Error("Invalid Record!");
	formcursor.first();
	form.setRecord(formcursor.primeUpdate());
	form.insert(projectform->title, "title");	
	form.insert(projectform->memo, "memo");
	form.insert(projectform->modified, "modified");
	form.insert(projectform->created, "created");
	form.readFields();

	projectform->publs->setEnabled(false);
    	projectform->publs->clear();
	fillpubl=new FillLinkListBox(this, "publfllb", projectform->publs, "select publication.no, publication.title from publication, project_bibliography where publication.no=project_bibliography.id_no and project_bibliography.tab_no=tabid('publication') and project_bibliography.project_no="+no(url()), "publication/publication", loadPixmap("showpublication.png"));

	fillpubl->start();
	threads.append(fillpubl);

	fillpart=new FillLinkListBox(this, "partfllb", projectform->publs, "select part.no, part.title||' In: \"'||publication.title||'\"' as title from publication, part, project_bibliography where part.publication_no=publication.no and project_bibliography.id_no=part.no and project_bibliography.tab_no=tabid('part') and project_bibliography.project_no="+no(url()), "part/part", loadPixmap("showpublication.png"));
	fillpart->start();
	threads.append(fillpart);


	fillfile=new FillLinkListBox(this, "filefllb", projectform->publs, "select partdata.no, partdata.filename||' from \"'||part.title||'\" In: \"'||publication.title||'\"' as title from publication, part, partdata, project_bibliography where partdata.part_no=part.no and part.publication_no=publication.no and project_bibliography.id_no=partdata.no and project_bibliography.tab_no=tabid('partdata') and project_bibliography.project_no="+no(url()), "file/partdata", loadPixmap("showpublication.png"));


	fillfile->start();
	threads.append(fillfile);

	setValid(true);
    } catch(Error &e) {
	setValid(false);
	setReadonlyGui(true);
	QMessageBox::warning(this, "Error", e.message());
	return false;
    }
    waitForThreads();
    setValid(true);
    //    projectform->memo->setText("<a href=\"http://www.test.de\">Test</a>");
    return true;
}

void Project::setWorkingGui(bool work) {
  ReadWriteFormBase::setInvalidGui(work);
  projectform->publs->setEnabled(!work);
 }

void Project::setReadonlyGui(bool ro) {
    ReadWriteFormBase::setReadonlyGui(ro);
    projectform->publs->setEnabled(!ro);
}

void Project::setInvalidGui(bool inv) {
    ReadWriteFormBase::setInvalidGui(inv);      
    setWorkingGui(false);
    setReadonlyGui(inv);
}

void Project::deletePublication() {
  write();
  LitUrl u=projectform->publs->url();
  QSqlQuery q("delete from project_bibliography where project_no="+no(url())+" and tab_no =tabid('"+u.table()+"') and id_no="+no(u));
  ASSERT(url().isValid());
  if(!setData())
    setInvalidGui(true);
}

void Project::generateBibTex() {
  GenerateBibTex *g=new GenerateBibTex(this);
  g->setProjectNo(no(url()));
  g->exec();
}

void Project::insertPublication() {
  SelectItem *s=new SelectItem(this, "selpublication");
  s->setData("publication/publication", "select publication.no, publication.title||' ('||author.firstname||' '||author.lastname||')' from author, publication_author, publication where author.no = publication_author.author_no and publication_author.publication_no=publication.no order by publication.title");
  if(s->exec()==QDialog::Accepted)
    createProjectBibliography(url(), s->url());
}

void Project::insertPart() {
  SelectPart *s=new SelectPart(this, "selpart");
  if(s->exec()==QDialog::Accepted)
    createProjectBibliography(url(), s->url());
}
