/***************************************************************************
 *                                                                         *
 *   publication.cpp								   *
 *									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Read the file COPYING for details.					   *
 *									   *
 *   copyright: (C) 2003 by Jan Mueller					   *
 *   email: janmueller7@hotmail.com					   *
 *									   *
 ***************************************************************************/

#include "publication.h"
#include "liturl.h"
#include <qlayout.h>
#include <qtoolbar.h>
#include <qaction.h>
#include <qpixmap.h>
#include <qmessagebox.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qapplication.h>
#include <qsqldatabase.h>
#include <qsqlcursor.h>
#include <qevent.h>
#include <qcombobox.h>
#include <qsettings.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include "error.h"
#include "editorhelper.h"
#include "global.h"
#include "linklistbox.h"
#include "authorlinklistbox.h"
#include "filllinklistbox.h"
#include "mainwindowimpl.h"
#include "urlevent.h"
#include "selectnote.h"
#include "selectpart.h"
#include "selectitem.h"
#include "bibtextypes.h"
#include <qvaluelist.h>

Publication::Publication(LitUrl url, QObject *receiver, QString slot, MainWindow *parent, const char *name): ReadWriteFormBase(url, receiver, slot, parent, name), toolbar(0L) {    
  publform=new PublicationForm(this, "publform");
  QHBoxLayout *l=new QHBoxLayout(this);
  l->addWidget(publform);
  
  connect(publform->chooseJournal, SIGNAL(clicked()), this, SLOT(chooseJournal()));
  connect(publform->choosePublisher, SIGNAL(clicked()), this, SLOT(choosePublisher()));
  connect(publform->bibtype, SIGNAL(activated(int)), this, SLOT(checkJournal()));

  QSettings settings;
  QString backgr = settings.readEntry(QString("/literature/backgr_")+name, "");
  if(!backgr.isEmpty()) publform->setErasePixmap(QPixmap(backgr));
  
  initToolBar();
  new EditorHelper(this, publform->memo);
  setData();
}

void Publication::initToolBar() {
  toolbar=new QToolBar(mainWindow());
  
  showBiblioAction=new QAction("Show Bibliography", QIconSet(loadPixmap("showbiblio.png")), "Show Bibliography", 0, this, "showbiblio");
  connect(showBiblioAction, SIGNAL(activated()), this, SLOT(showBiblio()));
  
  
  // **** Note ****
  insertNoteAction=new QAction("Insert Note", QIconSet(loadPixmap("insnote.png")), "Insert Note", 0, this, "insertnote");
  connect(insertNoteAction, SIGNAL(activated()), this, SLOT(insertNote()));
  
  createNoteAction=new QAction("Create Note", QIconSet(loadPixmap("newnote.png")), "Create Note", 0, this, "createnote");
  connect(createNoteAction, SIGNAL(activated()), this, SLOT(createNote()));
  
  deleteNoteAction=new QAction("Delete Note", QIconSet(loadPixmap("delnote.png")), "Delete Note", 0, this, "deletenote");
  connect(deleteNoteAction, SIGNAL(activated()), this, SLOT(deleteNote()));
  
  // **** Author ****
  insertAuthorAction=new QAction("Insert Author", QIconSet(loadPixmap("insauthor.png")), "Insert Author", 0, this, "insertpublication");
  connect(insertAuthorAction, SIGNAL(activated()), this, SLOT(insertAuthor()));
  createAuthorAction=new QAction("Create Author", QIconSet(loadPixmap("newauthor.png")), "Create Author", 0, this, "createpublication");
  connect(createAuthorAction, SIGNAL(activated()), this, SLOT(createAuthor()));
  deleteAuthorAction=new QAction("Delete Author", QIconSet(loadPixmap("delauthor.png")), "Delete Author", 0, this, "deletepublication");
  connect(deleteAuthorAction, SIGNAL(activated()), this, SLOT(deleteAuthor()));
  
  // **** Part ****
  createPartAction=new QAction("Create Part", QIconSet(loadPixmap("newpart.png")), "Create Part", 0, this, "createpart");
  connect(createPartAction, SIGNAL(activated()), this, SLOT(createPart()));
  
  deletePartAction=new QAction("Delete Part", QIconSet(loadPixmap("delpart.png")), "Delete Part", 0, this, "deletepart");
  connect(deletePartAction, SIGNAL(activated()), this, SLOT(deletePart()));

  addToProjectAction=new QAction("Add to Project", QIconSet(loadPixmap("gnome-folder.png")), "Add to Project", 0, this, "addtoproject");
  connect(addToProjectAction, SIGNAL(activated()), this, SLOT(addToProject()));

  
  showBiblioAction->addTo(toolbar);
  
  insertNoteAction->addTo(toolbar);
  createNoteAction->addTo(toolbar);
  deleteNoteAction->addTo(toolbar);
  
  insertAuthorAction->addTo(toolbar);
  createAuthorAction->addTo(toolbar);
  deleteAuthorAction->addTo(toolbar);
  
  createPartAction->addTo(toolbar);    
  deletePartAction->addTo(toolbar);

  addToProjectAction->addTo(toolbar);
  
  toolbar->show();

  showBiblioAction->addTo(roMenu());
  
  insertNoteAction->addTo(roMenu());
  createNoteAction->addTo(roMenu());
  deleteNoteAction->addTo(roMenu());
  
  insertAuthorAction->addTo(roMenu());
  createAuthorAction->addTo(roMenu());
  deleteAuthorAction->addTo(roMenu());
  
  createPartAction->addTo(roMenu());
  deletePartAction->addTo(roMenu());

  addToProjectAction->addTo(roMenu());
}

void Publication::write() {
  ReadWriteFormBase::write();
  if(!publform->authors->modified()) return;
  typedef QValueList<LitUrl> List;
  List l=publform->authors->authorPos();
  
  List::iterator it;
  int i=0;
  for ( it = l.begin(); it != l.end(); ++it ) {
    QString n=no(*it);
    QSqlQuery("update publication_author set author_order="+QString().setNum(i)+" where publication_no="+no(url())+" and author_no="+n);
    i++;
  }
}

Publication::~Publication() {
  write();
  delete toolbar;
}

void Publication::chooseJournal() {
  SelectItem *s=new SelectItem(this, "seljournal");
  Q_ASSERT(s);
  s->setData("journal/journal", "select no, name from journal order by name");
  if(s->exec()==QDialog::Accepted) {
    QSqlRecord *rec=formcursor.primeUpdate();
    rec->setValue("journal_no", no(s->url()));
    //    QSqlQuery query("update publication set journal_no="+no(s->url())+" where no="+no(url()));
    publform->journal->setText(s->text());
    debug(s->url());
  }
}

void Publication::choosePublisher() {
  SelectItem *s=new SelectItem(this, "selpublisher");
  Q_ASSERT(s);
  s->setData("publisher/publisher", "select no, name from publisher order by name");
  if(s->exec()==QDialog::Accepted) {
    QSqlRecord *rec=formcursor.primeUpdate();
    rec->setValue("publisher_no", no(s->url()));

    //    QSqlQuery query("update publication set publisher_no="+no(s->url())+" where no="+no(url()));
    publform->publisher->setText(s->text());
  }
}

bool Publication::setData()
{
  try {
    ReadWriteFormBase::setData();
    LitUrl u=url();
    setWorkingGui(true);

    int i;
    publform->bibtype->clear();
    for(i=0; i<bibtextypecount; i++)
      publform->bibtype->insertItem(bibtexnames[i]);
    
    form.clear();
    formcursor.clear();
    formcursor.setMode(QSqlCursor::Writable);
    formcursor.setName("publication");
    if(!formcursor.select(u.query()))
      throw(Error(formcursor.lastError()));
    qDebug("count:"+QString().setNum(formcursor.size()));
    if(formcursor.size()!=1) throw Error("Invalid Record!");
    formcursor.first();
    QString no=formcursor.value("no").toString();
    form.setRecord(formcursor.primeUpdate());
    form.insert(publform->title, "title");
    form.insert(publform->year, "year");
    form.insert(publform->bibtype, "entrytype");
    form.insert(publform->memo, "memo");
    form.insert(publform->modified, "modified");
    form.insert(publform->created, "created");
    form.readFields();

    checkJournal();

    // int entrytype=formcursor.value("entrytype").toInt();
    // publform->bibtype->setCurrentItem(entrytype);

    QString journal=formcursor.value("journal_no").toString();
    QString publisher=formcursor.value("publisher_no").toString();
        
    QSqlQuery query("select journal.name from journal where no="+journal);
    if(query.first())
      publform->journal->setText(query.value(0).toString());
    
    query.exec("select publisher.name from publisher where no="+publisher);
    if(query.first())
      publform->publisher->setText(query.value(0).toString());
    
        
    publform->links->setEnabled(false);
    publform->authors->setEnabled(false);
    publform->parts->setEnabled(false);
    
    publform->authors->clear();
    fillauthor=new FillLinkListBox(this, "publauthorfllb", publform->authors, "select author.no, author.firstname, author.lastname from author, publication_author where author.no=publication_author.author_no and publication_author.publication_no="+no+" order by publication_author.author_order desc", "author/author", loadPixmap("showauthor"));
    fillauthor->start();
    threads.append(fillauthor);
    
    publform->parts->clear();
    fillpart=new FillLinkListBox(this, "partpublicationfllb", publform->parts, "select part.no, part.partno || ' - ' || part.title from part, publication where part.publication_no=publication.no and publication.no="+no+" order by part.partno desc","part/part", loadPixmap("showpart.png"));
    fillpart->start();
    threads.append(fillpart);
    
    publform->links->clear();
    filllink=new FillLinkListBox(this, "linkpublicationfllb", publform->links, "select note.no, note.title from note, link where"+buildLinkWhereClause("note", "publication", no), "note/note", loadPixmap("shownote.png"));
    filllink->start();
    threads.append(filllink);
    
    setValid(true);
  } catch(Error &e) {
    setValid(false);
    setReadonlyGui(true);
    QMessageBox::warning(this, "Error", e.message());
    return false;
  }
  waitForThreads();
  setValid(true);
  return true;
}

void Publication::setWorkingGui(bool work) {
  ReadWriteFormBase::setWorkingGui(work);
  publform->links->setEnabled(!work);
  publform->authors->setEnabled(!work);
  publform->parts->setEnabled(!work);

}

void Publication::setReadonlyGui(bool ro) {
  ReadWriteFormBase::setReadonlyGui(ro);
  publform->title->setEnabled(!ro);
  publform->year->setEnabled(!ro);
  publform->bibtype->setEnabled(!ro);
  publform->memo->setEnabled(!ro);
}

void Publication::setInvalidGui(bool inv) {
  ReadWriteFormBase::setInvalidGui(inv);  
  setWorkingGui(false);
  setReadonlyGui(inv);
}

void Publication::showBiblio() {
  qApp->postEvent(topLevelWidget(), new UrlEvent(LitUrl("lit://bibliography/publication?no="+formcursor.value("no").toString()), false));
}

void Publication::addToProject() {
  SelectItem *s=new SelectItem(this, "selproject");
  Q_ASSERT(s);
  s->setData("project/project", "select no, title from project order by title");
  if(s->exec()==QDialog::Accepted) {
    QSqlQuery q("insert into project_bibliography (project_no, tab_no, id_no) values ("+
		no(s->url())+",tabid('publication'), "+no(url())+")");
  }
}

void Publication::insertNote() {
  SelectItem *s=new SelectItem(this, "selnote");
  Q_ASSERT(s);
  s->setData("note/note", "select no, title from note where publication_no="+no(url())+" order by title");
  if(s->exec()==QDialog::Accepted)
    createLink(s->url(), url());
}

void Publication::deleteNote() {
  deleteLink(publform->links->url(),url());    
}

void Publication::createNote() {
  newNote(url());
}

void Publication::insertAuthor() {
  SelectItem *s=new SelectItem(this, "selauthor");
  Q_ASSERT(s);
  s->setData("author/author", "select no, lastname, firstname from author order by lastname");
  if(s->exec()==QDialog::Accepted)
    createPublicationAuthor(url(), s->url());
}

void Publication::deleteAuthor() {
  deletePublicationAuthor(url(), publform->authors->url());
}

void Publication::createAuthor() {
  QSqlQuery q("select create_author() as no");
  q.first();
  LitUrl u("lit://author/author?no="+q.value(0).toString());
  createPublicationAuthor(url(), u);
  qApp->postEvent(topLevelWidget(), new UrlEvent(u, false));
}

void Publication::createPart() {
  QSqlQuery q("select create_part("+no(url())+") as no");
  q.first();
  LitUrl u("lit://part/part?no="+q.value(0).toString());
  qApp->postEvent(topLevelWidget(), new UrlEvent(u, false));
}

void Publication::deletePart() {
  if(!publform->parts->currentItem()) return;
  if(QMessageBox::information( this, "Literature",
			       "Do you want to delete the part?",
			       "&Yes", "&No")!=0) return;
  
  QSqlQuery query( "select del_obj(no) from "+publform->parts->url().table()+" where "+publform->parts->url().query());
  query.first();
  if(query.value(0).toString()!="true")
    QMessageBox::warning(this, "Error", "Can't delete part since records are locked!"+query.value(0).toString());
  setData();
  
}

void Publication::checkJournal() {
  /*
    #define BT_BOOK           0     // ->INBOOK
    #define BT_BOOKLET        1
    #define BT_JOURNAL        2	// ->ARTICLE
    #define BT_MANUAL         3
    #define BT_MASTERTHESIS   4
    #define BT_MISC           5
    #define BT_PROCEEDING     6	// ->INPROCEEDING
    #define BT_TECHREPORT     7
    #define BT_UNPUBLISHED    8
    #define BT_PHDTHESIS	9
    #define BT_INBOOK	        10
    #define BT_INCOLLECTION	11
    #define BT_INPROCEEDING	12
    #define BT_ARTICLE    	13
    
  */
  debug(QString("checkJournal:"+publform->bibtype->currentItem()));
  switch(publform->bibtype->currentItem()) {
  case BT_JOURNAL:
  case BT_ARTICLE:
    publform->chooseJournal->setEnabled(true);
    break;
  default:
    publform->chooseJournal->setEnabled(false);
  }
}
