/***************************************************************************
 *                                                                         *
 *   readonlyformbase.h								   *
 *									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Read the file COPYING for details.					   *
 *									   *
 *   copyright: (C) 2003 by Jan Mueller					   *
 *   email: janmueller7@hotmail.com					   *
 *									   *
 ***************************************************************************/

#ifndef ReadOnlyFormBase_h
#define ReadOnlyFormBase_h
#include <qwidget.h>
#include <qmainwindow.h>
#include <qsqlform.h>
#include "liturl.h"
#include <qsqlcursor.h>
#include <qsqldatabase.h>

/**
  @short Base class for read only dialogs.
  
  Base class for read only dialogs. Ensures that the dialog is destructed when closed
  and offers tools like a pointer to the main window, the url, the database connection,
  a pre-configured cursor for convenience and a valid/invalid state mechanism.
  Each instance of the app has its own database connection. 
  It can be accessed via conn(). A cursor "formcursor"  has been created for convenience.
  Don't use the default connection (ensures locking).
*/
class QToolBar;
class QPopupMenu;
class QAction;
class ReadOnlyFormBase: public QWidget {
    Q_OBJECT
   
private:
	ReadOnlyFormBase(const ReadOnlyFormBase&);
	ReadOnlyFormBase& operator=(const ReadOnlyFormBase&);
	QPopupMenu *romenu;	
	bool valid;
	
protected:
	LitUrl _url;
	QSqlForm form;
	QSqlCursor formcursor;
	QToolBar *rotoolbar;
	/** Returns the database connection of this instance.
	  */
	QAction *selBackgrAction;
	QAction *reloadRecAction;
	void initToolBar();
	virtual bool setData() { return true; };
	QPopupMenu *roMenu();
	
public slots:
	virtual void selBackgr();	
	virtual void reloadRec();

public:
	/** Constructor. Ensures that the dialog is destructed when closed. 
	 */
	ReadOnlyFormBase(LitUrl u, QObject *receiver, QString slot, QWidget *parent=0, const char *name=0): QWidget(parent, name,Qt::WDestructiveClose), romenu(0L), valid(false), _url(u), rotoolbar(0L) {
	  connect(this, SIGNAL(workingGui(bool)), receiver, slot);
	  initToolBar();
	};

	~ReadOnlyFormBase() { delete rotoolbar; };
	bool isValid() { return valid; }
	/** Returns a pointer to the top level widget. Typecasted to QMainWindow.
	 */
	QMainWindow *mainWindow() {
	  QMainWindow *m=dynamic_cast<QMainWindow*>(topLevelWidget());
	  ASSERT(m);
	  return m;
	}

	QString buildLinkWhereClause(QString target, QString source, QString no);
	
             /** Returns the url of the form.
	  */
	LitUrl url() { return _url; };

	
protected slots:
	virtual void setValid(bool t) { valid=t; }
	QString no(LitUrl o);
	QString tab(LitUrl o);

signals:
	/** Send out by setWorkingGui().
	  */
	virtual void workingGui(bool);
	/** Send out by setReadOnlyGui().
	  */
	virtual void readonlyGui(bool);
	/** Send out by setInvalidGui().
	  */
	virtual void invalidGui(bool);

};

#endif
