/***************************************************************************
 *                                                                         *
 *   readwriteformbase.cpp								   *
 *									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Read the file COPYING for details.					   *
 *									   *
 *   copyright: (C) 2003 by Jan Mueller					   *
 *   email: janmueller7@hotmail.com					   *
 *									   *
 ***************************************************************************/

#include "readwriteformbase.h"
#include <qwidget.h>
#include <error.h>
#include <qmessagebox.h>
#include <qsqlcursor.h>
#include <qaction.h>
#include <qapplication.h>
#include <qsettings.h>
#include <qpixmap.h>
#include <qfiledialog.h>
#include <qlabel.h>
#include <qintdict.h>
#include "global.h"
#include "mainwindowimpl.h"
#include "linklistboxitem.h"
#include "linklistbox.h"
#include "filllinklistbox.h"
#include "threadlinkevent.h"
#include "urlevent.h"
#include <qobjectlist.h>

ReadWriteFormBase::ReadWriteFormBase(LitUrl u, QObject *receiver, QString slot, QWidget *parent, const char *name): ReadOnlyFormBase(u, receiver, slot, parent, name), rec_is_locked(false), valid(false) {
  initToolBar();
  connect(&threadtimer, SIGNAL(timeout()), SLOT(checkThreads()));
  threadtimer.start(200, false);
}

ReadWriteFormBase::~ReadWriteFormBase() {
  waitForThreads();
  unlockRec();
  delete rwtoolbar;
}

bool ReadWriteFormBase::threadsFinished() {
  FillLinkListBox *f;
  for(f=threads.first(); f; f=threads.next())
    if(!f->finished()) return false;
  return true;
}

void ReadWriteFormBase::waitForThreads() {
  FillLinkListBox *f;
  unsigned int i;
  do {
    i=0;
    for(f=threads.first(); f; f=threads.next())
      if(f->wait(100)) i++;
    qApp->processEvents();
    
    qDebug("ReadWriteFormBase::waitForThreads() - i="+QString().setNum(i));
    qDebug("threads.count()="+QString().setNum(threads.count()));
  } while(threads.count()>i);
}

void ReadWriteFormBase::checkThreads() {
  if(threadsFinished()) {
    setWorkingGui(false);
    //    threadtimer.stop();
  }
}

/*
  Locking:
  Locking has two levels:
  external locking: Can this process lock the tuple or is it already locked 
  by another process?
  internal locking: Is the tuple already locked by this process and 
  displayed in another window?
  */
bool ReadWriteFormBase::lockRec() {
  qDebug("lockRec()");
  if(rec_is_locked) {
    setReadonlyGui(false);
    return true;
  }
    
  // Check whether the tuple is internaly locked by another window...
  bool soft_locked=locked_recs_dict.find(no(url()).toLong());
  if(soft_locked) {
    qDebug("ReadWriteFormBase::lockRec(): tuple locked by other window.");	   
    setReadonlyGui(true);
    return false;	
  }
    
  // Check whether the tuple is locked by another process...
  QSqlQuery query( "select lock_tuple(oid) from "+url().table()+" where "+url().query());
  query.first();
  bool db_locked=(query.value(0).toString()=="false");
  if(db_locked) {
    qDebug("ReadWriteFormBase::lockRec(): tuple locked by another process.");
    setReadonlyGui(true);
    return false;
  }
    
  qDebug("ReadWriteFormBase::lockRec(): tuple not locked by another process or another window.");
    
  locked_recs_dict.insert(no(url()).toLong(), new bool(true));
  setReadonlyGui(false);
  return true;
}
 
void ReadWriteFormBase::unlockRec() {
  if(!rec_is_locked) return;
  locked_recs_dict.remove(no(url()).toLong());
  QSqlQuery query( "select unlock_tuple(oid) from "+url().table()+" where "+url().query());
  rec_is_locked=false;
}

void ReadWriteFormBase::lockGui() {
  const QObjectList *list=children();
	
  QObjectListIt it(*list);
  for ( ; it.current(); ++it ) {
    if(it.current()->isWidgetType()) {
      QWidget *w=dynamic_cast<QWidget*>(it.current());
      w->setEnabled(false);
    }
  }    
}

void ReadWriteFormBase::unlockGui() {
  const QObjectList *list=children();
	
  QObjectListIt it(*list);
  for ( ; it.current(); ++it ) {
    if(it.current()->isWidgetType()) {
      QWidget *w=dynamic_cast<QWidget*>(it.current());
      w->setEnabled(true);
    }
  }    
}

void ReadWriteFormBase::copy() {
}

void ReadWriteFormBase::cut() {
}

void ReadWriteFormBase::paste() {
}


void ReadWriteFormBase::write()
{ 
  if(!isValid()) return;
  if(formcursor.position("created")>-1) formcursor.remove(formcursor.position("created"));
  if(formcursor.position("modified")>-1) formcursor.remove(formcursor.position("modified"));

  form.writeFields();
  if(formcursor.primaryIndex().isEmpty()) throw Error("No primary Index!");
  if(!formcursor.update())
    QMessageBox::warning(this, "Error", "Can't update database!");
}

void ReadWriteFormBase::deleteRec()
{
  if(!isValid()) return;
  if(QMessageBox::information( this, "Literature",
			       "Do you want to delete the record?",
			       "&Yes", "&No")!=0) return;
	    	
  if(!rec_is_locked) return;
  QSqlQuery query( "select del_obj(no) from "+url().table()+" where "+url().query());
  query.first();
  if(query.value(0).toString()!="true")
    QMessageBox::warning(this, "Error", "Can't delete record since one or more depending records are locked!"+query.value(0).toString());
  else {
    setInvalidGui(true);
    setValid(false);
  }
}

void ReadWriteFormBase::setWorkingGui(bool work) 
{
  rwtoolbar->setEnabled(!work);
  emit workingGui(work);
}

void ReadWriteFormBase::setReadonlyGui(bool ro)
{
  debug("ReadWriteFormBase::setReadonlyGui(bool ro)");
  emit readonlyGui(ro);
}

void ReadWriteFormBase::setInvalidGui(bool inv) 
{
  writeAction->setEnabled(!inv);
  deleteAction->setEnabled(!inv);
  selBackgrAction->setEnabled(!inv);
  emit invalidGui(inv);
}

void ReadWriteFormBase::customEvent(QCustomEvent *e) {
  ThreadLinkEvent *l=dynamic_cast<ThreadLinkEvent*>(e);
  if(l && l->parentItem())
    new LinkListBoxItem(l->parentItem(), l->text(), l->url(), l->pixmap());
  else if(l) 
    new LinkListBoxItem(l->box(), l->text(), l->url(), l->pixmap());
}

bool ReadWriteFormBase::setData(void) {
  // we can't call lockRec() in the constructor since it calls setxxxxGui(), which is virtual und reimplemented by derived classes and not vailable in the constructor.
  rec_is_locked=lockRec();
  return true;
}

void ReadWriteFormBase::initToolBar() {
  rwtoolbar=new QToolBar(mainWindow());
    
  writeAction=new QAction("Write", QIconSet(loadPixmap("floppy.png")), "&Write", Qt::CTRL+Key_W, this, "write");
  connect(writeAction, SIGNAL(activated()), this, SLOT(write()));
  deleteAction=new QAction("Delete", QIconSet(loadPixmap("delete.png")), "&Delete", Qt::CTRL+Key_D, this, "delete");
  connect(deleteAction, SIGNAL(activated()), this, SLOT(deleteRec()));
  writeAction->addTo(rwtoolbar);
  deleteAction->addTo(rwtoolbar);
  rwtoolbar->show();

  writeAction->addTo(roMenu());
  deleteAction->addTo(roMenu());
}

void ReadWriteFormBase::createLink(LitUrl obj1, LitUrl obj2) {
  write();
  QSqlQuery q("insert into link (obj_1, tab_1, obj_2, tab_2) values ("+no(obj1)+",tabid('"+obj1.table()+"'),"+no(obj2)+",tabid('"+obj2.table()+"'))");
  ASSERT(url().isValid());
  if(!setData())
    setInvalidGui(true);
}


void ReadWriteFormBase::createPublicationAuthor(LitUrl publ, LitUrl author) {
  write();
  QSqlQuery q("select max(author_order) from publication_author where publication_no="+no(publ));
  q.first();
  int o=q.value(0).toInt();
  q.exec("insert into publication_author (publication_no, author_no, author_order) values ("+no(publ)+","+no(author)+","+QString().setNum(o+1)+")");
  ASSERT(url().isValid());
  if(!setData())
    setInvalidGui(true);
}

void ReadWriteFormBase::createProjectBibliography(LitUrl pr, LitUrl pub) {
  write();
  QSqlQuery q("insert into project_bibliography (project_no, id_no, tab_no) values ("+no(pr)+","+no(pub)+","+tab(pub)+")");
  ASSERT(url().isValid());
  if(!setData())
    setInvalidGui(true);
}

void ReadWriteFormBase::deleteProjectBibliography(LitUrl pr, LitUrl publ) {
  write();
  QSqlQuery q("delete from project_bibliography where project_no="+no(publ)+" and id_no="+no(publ)+" and tab_no="+tab(publ)+")");
  ASSERT(url().isValid());
  if(!setData())
    setInvalidGui(true);
}


void ReadWriteFormBase::createPartAuthor(LitUrl part, LitUrl author) {
  write();
  QSqlQuery q("select max(author_order) from part_author where part_no="+no(part));
  q.first();
  int o=q.value(0).toInt();
  
  q.exec("insert into part_author (part_no, author_no, author_order) values ("+no(part)+","+no(author)+","+QString().setNum(o+1)+")");
  ASSERT(url().isValid());
  if(!setData())
    setInvalidGui(true);
}

void ReadWriteFormBase::deletePublicationAuthor(LitUrl publ, LitUrl author) {
  write();
  QSqlQuery q("delete from publication_author where publication_no="+no(publ)+" and author_no="+no(author));
  ASSERT(url().isValid());
  if(!setData())
    setInvalidGui(true);
}

void ReadWriteFormBase::deletePartAuthor(LitUrl part, LitUrl author) {
  write();
  QSqlQuery q("delete from part_author where part_no="+no(part)+" and author_no="+no(author));
  ASSERT(url().isValid());
  if(!setData())
    setInvalidGui(true);
}

void ReadWriteFormBase::deleteLink(LitUrl obj1, LitUrl obj2) {
  write();
  QSqlQuery q("delete from link where (obj_1="+no(obj1)+" and obj_2="+no(obj2)+" and tab_1=tabid('"+obj1.table()+"') and tab_2=tabid('"+obj2.table()+"')) or (obj_2="+no(obj1)+" and obj_1="+no(obj2)+" and tab_2=tabid('"+obj1.table()+"') and tab_1=tabid('"+obj2.table()+"'))");
  ASSERT(url().isValid());
  if(!setData())
    setInvalidGui(true);
}

void ReadWriteFormBase::newNote(LitUrl obj) {
  QSqlQuery q("select create_publ_note("+no(obj)+") as no");
  q.first();
  QString n=q.value(0).toString();
  q.exec("insert into link (obj_1, tab_1, obj_2, tab_2) values ("+n+",tabid('note'),"+no(url())+",tabid('"+url().table()+"'))");

  qApp->postEvent(topLevelWidget(), new UrlEvent(LitUrl("lit://note/note?no="+n), false));
}
 
